/*
	Copyright (c) 2004 Jonatan Kronqvist

	Permission is hereby granted, free of charge, to any person obtaining a
	copy of this software and associated documentation files (the "Software"),
	to deal in the Software without restriction, including without limitation
	the rights to use, copy, modify, merge, publish, distribute, sublicense,
	and/or sell copies of the Software, and to permit persons to whom the
	Software is furnished to do so, subject to the following conditions:

	The above copyright notice and this permission notice shall be included in
	all copies or substantial portions of the Software.

	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
	OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
	FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
	AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
	LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
	FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
	IN THE SOFTWARE.
*/


#include "foobar_control.h"


void foobar_control::fade_stop()
{
	// TODO: fix the actual fading :)
	stop();
}


void foobar_control::ffwd(long secs)
{
	// TODO: figure out if the param is in seconds
	// also, where the hell in the client can we seek? :)
	play_control::get()->playback_seek_delta(secs);
}


void foobar_control::rwnd(long secs)
{
	// TODO: figure out if the param is in seconds
	// also, where the hell in the client can we seek? :)
	play_control::get()->playback_seek_delta(-secs);
}

unsigned char foobar_control::get_volume()
{
	int dBvol = play_control::get()->get_volume();	// in 0.01 dB increments
	return (unsigned char)(256.0f / (dBvol/100.0f+100.0f));	// loose some precision -> -100 .. 0 dB
}

void foobar_control::set_volume(unsigned char vol)
{
	int dBvol = int((vol/256.0f * 10000.0f) - 10000.0f);	// 0.01 dB increments
	// because of loss of precision:
	if(vol == 255) dBvol = 0;
	play_control::get()->set_volume(dBvol);
}

void foobar_control::add_to_playlist(string8 fname)
{
	string8 url ("file://");
	url.add_string(fname);
	g_debug(url);
	playlist_oper::get()->add_location(url);
}


void foobar_control::next(bool random)
{
	if(m_shuffle && random)
		play_control::get()->play_start(play_control::TRACK_COMMAND_RAND);
	else
		play_control::get()->play_start(play_control::TRACK_COMMAND_NEXT);
}


void foobar_control::prev()
{
	play_control::get()->play_start(play_control::TRACK_COMMAND_PREV);
}


void foobar_control::pause()
{
	play_control::get()->pause();
}



void foobar_control::stop()
{
	play_control::get()->play_stop();
}


void foobar_control::play(string8 fname)
{
	string8 url ("file://");
	url.add_string(fname);
	g_debug(url);

	// replace the playlist and play the first (if dir) entry
	playlist_oper::get()->add_location(url, true);
	playlist_oper::get()->play_item(0);
}


void foobar_control::play_current()
{
	play_control::get()->play_start();
}

long foobar_control::get_playlist_len()
{
	return playlist_oper::get()->get_count();
}

std::vector<string8> *foobar_control::get_playlist()
{
	std::vector<string8> *ret = new std::vector<string8>;

	metadb_handle *handle;
	unsigned short playlist_len = (unsigned short)get_playlist_len();
	for(int i=0; i<playlist_len; i++) {
		handle = playlist_oper::get()->get_item(i);
		string8 title;
		handle->handle_format_title(title, m_playlist_fmt, NULL);
		title.add_char('\n');
		ret->push_back(title);
		handle->handle_release();
	}
	return ret;
}

void foobar_control::select_at(int idx)
{
	string8 str ("Selected track for play at: ");
	str.add_int(idx);
	g_debug(str);
	
	playlist_oper::get()->set_sel_one(idx);
	playlist_oper::get()->play_item(idx);
}


void foobar_control::clear_playlist()
{
	playlist_oper::get()->remove_all();
}


void foobar_control::set_repeat(bool repeat)
{
	if(repeat)
		playback_order_helper::set_config_by_name("Repeat");
	else
		playback_order_helper::set_config_by_name("Default");
}


void foobar_control::set_shuffle(bool shuffle)
{
	m_shuffle = shuffle;	// checked in bluetooth_control::next()
	if(shuffle)
		playback_order_helper::set_config_by_name("Random");
	else
		playback_order_helper::set_config_by_name("Default");
}


void foobar_control::seek(long pos)
{
	if(play_control::get()->playback_can_seek())
		play_control::get()->playback_seek((double)pos);
}


void foobar_control::set_stop_at_eof_track(bool stop)
{
	play_control::get()->set_stop_after_current(stop);
}


long foobar_control::get_bitrate()
{
	file_info_i_full finf;
	metadb_handle *h = play_control::get()->get_now_playing();
	h->handle_lock();
	h->handle_query(&finf);
	long bitrate = (long)finf.info_get_bitrate();
	h->handle_unlock();
	h->handle_release();

	return bitrate;
}

long foobar_control::get_samplerate()
{
	file_info_i_full finf;
	metadb_handle *h = play_control::get()->get_now_playing();
	h->handle_lock();
	h->handle_query(&finf);

	audio_chunk_i chunk;
	input_helper in;
	in.open(h);
	in.run(&chunk);
	long samplerate = chunk.get_srate();
	in.close();

	h->handle_unlock();
	h->handle_release();
	
	return samplerate;
}

long foobar_control::get_channels()
{
	file_info_i_full finf;
	metadb_handle *h = play_control::get()->get_now_playing();
	h->handle_lock();
	h->handle_query(&finf);

	audio_chunk_i chunk;
	input_helper in;
	in.open(h);
	in.run(&chunk);
	long channels = chunk.get_channels();
	in.close();

	h->handle_unlock();
	h->handle_release();

	return channels;
}

const char* foobar_control::get_file_path()
{
	file_info_i_full finf;
	metadb_handle *h = play_control::get()->get_now_playing();
	h->handle_lock();
	h->handle_query(&finf);
	h->handle_unlock();
	h->handle_release();

	return finf.get_file_path();
}

long foobar_control::get_playback_time()
{
	return (long)play_control::get()->get_playback_time();
}

long foobar_control::get_playlength()
{
	return (long)play_control::get()->playback_get_length();
}

bool foobar_control::get_shuffle()
{
	string8 tmpstr;
	playback_order_helper::get_config_by_name(tmpstr);
	m_shuffle = (tmpstr.find_first("Random") != -1);
	return m_shuffle;
}

bool foobar_control::get_repeat()
{
	string8 tmpstr;
	playback_order_helper::get_config_by_name(tmpstr);
	return (tmpstr.find_first("Repeat") != -1);
}

void foobar_control::get_title(string8 &str)
{
	play_control::get()->playback_format_title(str, m_playlist_fmt);
}

bool foobar_control::is_paused()
{
	return play_control::get()->is_paused();
}

bool foobar_control::is_playing()
{
	return play_control::get()->is_playing();
}

long foobar_control::get_cur_idx()
{
	return playlist_oper::get()->get_now_playing();
}


void foobar_control::set_playlist_fmt(string8 playlist_fmt)
{
	m_playlist_fmt = playlist_fmt;
}
